@@ -13,22 +13,21 @@
 ## along with Lino; if not, see <http://www.gnu.org/licenses/>.
 
 """
-Lino is a Python package to be used on Django sites.
-See :doc:`/admin/install` on how to use it.
+The ``lino`` module can be imported even from a Django :xfile:`settings.py` 
+file since it does not import any django module.
 
 """
 
-# NOTE: this module may not import any django module since it is being imported from settings.py files
 import os
 import sys
 import datetime
 from os.path import join, abspath, dirname, normpath
 
 
-__version__ = "1.3.9"
+__version__ = "1.4.5"
 """
 Lino version number. 
-The latest *released* version is :doc:`/releases/2012/1.3.8`.
+*Released* versions are listed under :doc:`/releases`.
 """
 
 __author__ = "Luc Saffre <luc.saffre@gmx.net>"
@@ -90,15 +89,14 @@ if False:
 
 NOT_FOUND_MSG = '(not installed)'
 
-def using():
+def using(ui=None):
     """
     Yields a list of third-party software descriptors used by Lino.
     Each descriptor is a tuple (name, version, url).
     
     """
-    import sys
-    version = "%d.%d.%d" % sys.version_info[:3]
-    yield ("Python",version,"http://www.python.org/")
+    
+    yield ("Lino",__version__,"http://lino.saffre-rumma.net")
     
     import django
     yield ("Django",django.get_version(),"http://www.djangoproject.com")
@@ -114,6 +112,13 @@ def using():
         version = NOT_FOUND_MSG
     yield ("Cheetah",version ,"http://cheetahtemplate.org/")
 
+    try:
+        from odf import opendocument
+        version = opendocument.__version__
+    except ImportError:
+        version = NOT_FOUND_MSG
+    yield ("OdfPy",version ,"http://pypi.python.org/pypi/odfpy")
+
     try:
         import docutils
         version = docutils.__version__
@@ -121,6 +126,13 @@ def using():
         version = NOT_FOUND_MSG
     yield ("docutils",version ,"http://docutils.sourceforge.net/")
 
+    try:
+        import suds
+        version = suds.__version__
+    except ImportError:
+        version = NOT_FOUND_MSG
+    yield ("suds",version ,"https://fedorahosted.org/suds/")
+
     import yaml
     version = getattr(yaml,'__version__','')
     yield ("PyYaml",version,"http://pyyaml.org/")
@@ -133,15 +145,20 @@ def using():
             version = NOT_FOUND_MSG
         yield ("pyratemp",version,"http://www.simple-is-better.org/template/pyratemp.html")
     
-    try:
-        import ho.pisa as pisa
-        version = getattr(pisa,'__version__','')
-        yield ("xhtml2pdf",version,"http://www.xhtml2pdf.com")
-    except ImportError:
-        pass
+    if False:
+        try:
+            import ho.pisa as pisa
+            version = getattr(pisa,'__version__','')
+            yield ("xhtml2pdf",version,"http://www.xhtml2pdf.com")
+        except ImportError:
+            pass
 
-    import reportlab
-    yield ("ReportLab Toolkit",reportlab.Version, "http://www.reportlab.org/rl_toolkit.html")
+        try:
+            import reportlab
+            version = reportlab.Version
+        except ImportError:
+            version = NOT_FOUND_MSG
+        yield ("ReportLab",version,"http://www.reportlab.org/rl_toolkit.html")
                
     try:
         #~ import appy
@@ -149,18 +166,31 @@ def using():
         version = version.verbose
     except ImportError:
         version = NOT_FOUND_MSG
-    yield ("appy.pod",version ,"http://appyframework.org/pod.html")
+    yield ("Appy",version ,"http://appyframework.org/pod.html")
+    
+    import sys
+    version = "%d.%d.%d" % sys.version_info[:3]
+    yield ("Python",version,"http://www.python.org/")
+    
+    if ui is not None:
+        #~ version = '<script type="text/javascript">document.write(Ext.version);</script>'
+        onclick = "alert('ExtJS client version is ' + Ext.version);"
+        tip = "Click to see ExtJS client version"
+        text = "(version)"
+        version = """<a href="#" onclick="%s" title="%s">%s</a>""" % (onclick,tip,text)
+        yield ("ExtJS",version ,"http://www.sencha.com")
 
 
 def welcome_text():
-    return "Lino version %s using %s" % (
-      __version__, 
-      ', '.join(["%s %s" % (n,v) for n,v,u in using()]))
+    #~ return "Lino version %s using %s" % (
+    return "Using %s." % (', '.join(["%s %s" % (n,v) for n,v,u in using()]))
 
-def welcome_html():
-    return "Lino version %s using %s" % (
-      __version__,
-      ', '.join(['<a href="%s" target="_blank">%s</a> %s' % (u,n,v) for n,v,u in using()]))
+def welcome_html(ui=None):
+    #~ return "Lino version %s using %s" % (
+      #~ __version__,
+      #~ ', '.join(['<a href="%s" target="_blank">%s</a> %s' % (u,n,v) for n,v,u in using(ui)]))
+    sep = '<br/>'
+    return sep.join(['<a href="%s" target="_blank">%s</a> %s' % (u,n,v) for n,v,u in using(ui)])
 
 
 class Lino(object):
@@ -170,7 +200,7 @@ class Lino(object):
     Subclasses of this can be defined and instantiated in Django settings files.
     
     This class is first defined in :mod:`lino`, then subclassed by 
-    :mod:`lino.apps.dsbe.settings` or 
+    :mod:`lino.apps.pcsw.settings` or 
     :mod:`lino.apps.igen.settings`,
     which is imported into your local :xfile:`settings.py`,
     where you may subclass it another time.
@@ -187,11 +217,33 @@ class Lino(object):
     
     """
     
-    auto_makeui = True
+    never_build_site_cache = False
     """
-    useful when debugging directly on the generated `lino.js`
+    Set this to `True` if you want that Lino 
+    never (re)builds the site cache (even when asked). 
+    This can be useful on a development server when you are debugging 
+    directly on the generated :xfile:`lino*.js`.
+    Or for certain unit test cases.
     """
     
+    build_js_cache_on_startup = None
+    """
+    Whether the Javascript cache files should be built on startup 
+    for all user profiles and languages.
+    
+    On a production server this should be `True` for best performance,
+    but while developing it is easier to set it to `False`, which means 
+    that each file is built upon need (when a first request comes in).
+    
+    The default value `None` means that Lino decides automatically 
+    (using :func:`lino.core.modeltools.is_devserver`).
+    """
+    
+    # tree constants used by lino.modlib.workflows:
+    max_state_value_length = 20 
+    max_action_name_length = 50
+    max_actor_name_length = 100
+    
     root_url = '' # 
     """
     must begin with a slash if not empty
@@ -271,8 +323,10 @@ class Lino(object):
     model used as project in your application.
     """
     
-    user_model = "users.User"
-    """Set this to ``"auth.User"`` if you use `django.contrib.auth` instead of
+    #~ user_model = "users.User"
+    user_model = None
+    """
+    Set this to ``"auth.User"`` if you use `django.contrib.auth` instead of
     `lino.modlib.users`. 
     
     Set it to `None` to remove any user management 
@@ -285,15 +339,23 @@ class Lino(object):
     no REMOTE_USER header makes its way through to Lino. 
     Which may happen on a development server and if Apache is 
     configured to allow it.
-    Used by :mod:`lino.utils.auth`
-    :mod:`lino.modlib.users.middleware`
+    Used by :mod:`lino.utils.auth`.
     """
     
+    anonymous_user_profile = '900'
+    
+    
+    
     remote_user_header = "REMOTE_USER"
+    #~ remote_user_header = None
     """
     The name of the header (set by the web server) that Lino consults 
     for finding the user of a request.
+    Default value is ``"REMOTE_USER"``.
+    Settings this to `None` means that http authentication 
+    is not used at all.
     """
+    
     #~ simulate_remote_user = False
     
     
@@ -358,6 +420,19 @@ class Lino(object):
     Whether to use the `Extensible <http://ext.ensible.com>`_ calendar library.
     """
     
+    use_quicktips = True
+    """
+    Whether to make use of `Ext.QuickTips
+    <http://docs.sencha.com/ext-js/3-4/#!/api/Ext.QuickTips>`_
+    when displaying help texts defined in :class:`lino.models.HelpText`
+    """
+    
+    use_css_tooltips = False
+    """
+    Whether to make use of CSS tooltips
+    when displaying help texts defined in :class:`lino.models.HelpText`.
+    """
+    
     use_vinylfox = False
     """
     Whether to use VinylFox extensions for HtmlEditor. 
@@ -461,8 +536,8 @@ class Lino(object):
     
     loading_from_dump = False
     """
-    Set to `False` by dpy dumps that were generated by
-    :meth:`lino.utils.dpy.Serializer.serialize`.
+    Set to `False` by python dumps that were generated by
+    :meth:`lino.utils.dumpy.Serializer.serialize`.
     Used in 
     :func:`lino.modlib.cal.models.update_auto_task`
     and
@@ -474,7 +549,14 @@ class Lino(object):
     decimal_separator = ','
     """
     Set this to either ``'.'`` or ``','`` to define wether to use 
-    comma or dot as decimal separator when entering a `DecimalField`.
+    comma or dot as decimal point separator when entering 
+    a `DecimalField`.
+    """
+    
+    #~ decimal_group_separator = ','
+    decimal_group_separator = ' '
+    """
+    Decimal group separator for :func:`lino.utils.moneyfmt`.
     """
 
     
@@ -504,7 +586,7 @@ class Lino(object):
     
     alt_date_formats_extjs = 'd/m/Y|Y-m-d'
     """
-    Alternative date entry formats accepted by Date widgets.
+    Alternative date entry formats accepted by ExtJS Date widgets.
     """
     
     #~ date_format_regex = "/^[0123]\d\.[01]\d\.-?\d+$/"
@@ -526,25 +608,30 @@ class Lino(object):
     If you change this setting, you also need to override :meth:`parse_datetime`.
     """
     
-    bcss_soap_url = None
+    cbss_live_tests = False
     """
-    URL of BCSS SOAP server
+    Whether unit tests should try to really connect to the cbss.
+    Some test cases of the test suite would fail with a timeout if run 
+    from behind an IP address that is not registered at the :term:`CBSS`.
+    These tests are skipped by default. To activate them, 
+    set `cbss_live_tests` to `True` in your :xfile:`settings.py`.
+    
     """
     
-    bcss_user_params = None
+    unused_cbss_user_params = None
     u"""
-    User parameters for BCSS access.
+    User parameters for CBSS SSDN (classic) services.
     
     Example::
 
       class Lino(Lino):
           ...
-          bcss_user_params = dict(
+          cbss_user_params = dict(
                 UserID='123', 
                 Email='123@example.com', 
                 OrgUnit='123', 
-                MatrixID=12, 
-                MatrixSubID=3)
+                MatrixID=17, 
+                MatrixSubID=1)
 
     L’ « authorized user » est l’utilisateur de l’application. 
     Dans certains cas il s’agit de la personne physique derrière 
@@ -563,6 +650,30 @@ class Lino(object):
 
     """
     
+    cbss_environment = None
+    """
+    Either `None` or one of 'test', 'acpt' or 'prod'.
+    See :mod:`lino.modlib.cbss.models`.
+    Leaving this to `None` means that the cbss module is "inactive" even if installed.
+    """
+    
+    unused_cbss_cbe_number = '0123456789'
+    """
+    Either `None` or a string of style '0123456789'
+    Needed for CBSS new style services. See :mod:`lino.modlib.cbss.models`.
+    """
+    unused_cbss_username = None
+    """
+    Either `None` or a string of style 'E0123456789'
+    Needed for CBSS new style services. See :mod:`lino.modlib.cbss.models`.
+    """
+    unused_cbss_password = None
+    """
+    Either `None` or a string of style 'p1234567890abcd1234567890abcd'
+    Needed for CBSS new style services. See :mod:`lino.modlib.cbss.models`.
+    """
+    
+    
     use_davlink = False
     """
     Set this to `False` if you don't need WebDAV-enabled links.
@@ -594,12 +705,32 @@ class Lino(object):
     :class:`lino.models.SiteConfig` instance.
     """
     
+    max_auto_events = 36
+    """
+    Maximum number of automatic events to be generated 
+    by :class:`lino.modlib.cal.models.EventOwner`.
+    """
+    
+    #~ use_contenttypes = True
+    #~ """
+    #~ Set this to False if you don't want to use `django.contrib.contenttypes`.
+    #~ """
+    
+    #~ index_view_action = 'lino.Home'
+    
+    
+    
+    
     # for internal use:
+    
     _site_config = None
+    _extjs_ui = None
     
     
     def __init__(self,project_file,settings_dict):
       
+        #~ self.user_profile_fields = ['level']
+        
         self.project_dir = normpath(dirname(project_file))
         self.project_name = os.path.split(self.project_dir)[-1]
         
@@ -629,7 +760,13 @@ class Lino(object):
             self.webdav_root = join(abspath(self.project_dir),'media','webdav')
             
         settings_dict.update(MEDIA_ROOT = join(self.project_dir,'media'))
-        settings_dict.update(FIXTURE_DIRS = [join(self.project_dir,"fixtures")])
+        if self.project_dir != self.source_dir:
+            settings_dict.update(FIXTURE_DIRS = [join(self.project_dir,"fixtures")])
+            #~ lino.Lino.__init__ füllte project_dir auch dann nach FIXTURES_DIR, 
+            #~ wenn es zugleich das source_dir war. Was die subtile Folge hatte, 
+            #~ dass alle Fixtures doppelt ausgeführt wurden. 
+            #~ Dieser Bug hat mich mindestens eine Stunde lang beschäftigt.            
+
         settings_dict.update(TEMPLATE_DIRS = (
             join(abspath(self.project_dir),'templates'),
             join(abspath(self.source_dir),'templates'),
@@ -677,7 +814,45 @@ class Lino(object):
         #~ self.config = get_site_config()
         
 
+    def get_ui(self):
+        if self._extjs_ui is None:
+            self.startup()
+            from lino.ui.extjs3 import UI
+            self._extjs_ui = UI()
+        return self._extjs_ui
+    ui = property(get_ui)
 
+    override_modlib_models = []
+    """
+    A list of names of modlib models which are being 
+    redefined by this application.
+    
+    The following modlib models currently support this:
+    - :class:`contacts.Person <lino.modlib.contacts.models.Person>`
+    - :class:`contacts.Company <lino.modlib.contacts.models.Company>`
+    
+    Usage: in your application's `settings.py`, specify::
+    
+      class Lino(Lino):
+          override_modlib_models = ['contacts.Person']
+          
+    This will cause the modlib Person model to be abstract, 
+    and hence your application is responsible for defining anoter 
+    `Person` class with "contacts" as `app_label`::
+          
+    class Person(contacts.Person,contacts.Born):
+        class Meta(contacts.Person.Meta):
+            app_label = 'contacts'
+            
+        def kiss(self):
+            ...
+          
+    
+    """
+
+    def is_abstract_model(self,name):
+        return name in self.override_modlib_models
+        
     def parse_date(self,s):
         """Convert a string formatted using 
         :attr:`date_format_strftime` or  :attr:`date_format_extjs` 
@@ -733,6 +908,9 @@ class Lino(object):
         return __file__
         
     def setup_main_menu(self):
+        """
+        To be implemented by applications.
+        """
         pass
 
     #~ def update(self,**kw):
@@ -746,63 +924,196 @@ class Lino(object):
     #~ def configure(self,sc):
         #~ self.config = sc
         
-    def get_site_config(self):
-        from lino.models import get_site_config
-        return get_site_config()
-        #~ if self._site_config is None:
-            #~ from lino.models import get_site_config
-            #~ self._site_config = get_site_config()
-        #~ return self._site_config
-    site_config = property(get_site_config)
-        
-        
-    def has_module(self,name):
-        from django.conf import settings
-        return name in settings.INSTALLED_APPS
-        
-    def setup(self,**options):
+    def on_site_config_saved(self,sc):
         """
+        Called by SiteConfig.save()
+        """
+        self._site_config = sc
+        
+    def get_site_config(self):
+        """
+        Returns the one and only :class:`lino.models.SiteConfig` instance.
+        
+        If no instance exists (which happens in a virgin database),
+        we create it and set some default values from :attr:`site_config_defaults`.
+        """
+        if self._site_config is None:
+            from lino.models import SiteConfig
+            try:
+                self._site_config = SiteConfig.objects.get(pk=1)
+            except SiteConfig.DoesNotExist:
+            #~ except Exception,e:
+                kw = dict(pk=1)
+                #~ kw.update(settings.LINO.site_config_defaults)
+                kw.update(self.site_config_defaults)
+                #~ logger.debug("Creating SiteConfig record (%s)",e)
+                self._site_config = SiteConfig(**kw)
+                # do NOT save the instance here
+                # sc.save()
+        return self._site_config
+    site_config = property(get_site_config)
+    
+    def update_site_config(self,**kw):
+        """
+        Update and save the one and only :class:`lino.models.SiteConfig` instance.
+        """
+        sc = self.site_config
+        for k,v in kw.items():
+            setattr(sc,k,v)
+        sc.save()
+    
+        
+    def startup(self,**options):
+        """
+        Start the Lino site. 
+        This is called when Django has done his work 
+        (all models modules have been imported).
+        
         This is called whenever a user interface 
         (:class:`lino.ui.base.UI`) gets instantiated (which usually 
         happenes in some URLConf, for example in:mod:`lino.ui.extjs3.urls`). 
-        Also called by :term:`makedocs` with keyword argument `make_messages`.
+        Also called by some test cases.
         """
-        from lino.core.kernel import setup_site
-        setup_site(self,**options)
+        from lino.core.kernel import startup_site
+        startup_site(self,**options)
+        
+        
+    #~ def has_module(self,name):
+        #~ from django.conf import settings
+        #~ return name in settings.INSTALLED_APPS
+        
+    #~ def add_user_group(self,name,label):
+        #~ from lino.utils.choicelists import UserLevel, UserGroup
+        #~ UserGroup.add_item(name,label,name)
+        #~ self.add_user_field(name+'_level',UserLevel.field(label),profile=True)
+        
+    def setup_choicelists(self):
+        """
+        Redefine application-specific Choice Lists.
+        
+        Especially used to define application-specific
+        :class:`UserProfiles <lino.core.perms.UserProfiles>`.
+        
+        Lino by default has two user profiles "User" 
+        and "Administrator", defined in :mod:`lino.core.perms`.
+        
+        Application developers who use group-based requirements 
+        must override this in their application's :xfile:`settings.py` 
+        to provide a default list of user profiles for their 
+        application.
+        
+        See the source code of :mod:`lino.apps.presto` 
+        or :mod:`lino.apps.pcsw` for a usage example.
+        
+        Local site administrators may again override this in their 
+        :xfile:`settings.py`.
+        
+        """
+        pass
+        
+    def add_user_field(self,name,fld):
+        if self.user_model:
+            from lino import dd
+            User = dd.resolve_model(self.user_model)            
+            dd.inject_field(User,name,fld)
+            #~ if profile:
+                #~ self.user_profile_fields.append(name)
+
+    def is_installed(self,app_label):
+        """
+        Return `True` if :setting:`INSTALLED_APPS` contains an item
+        which ends with the specified `app_label`.
+        """
+        from django.conf import settings
+        if not '.' in app_label:
+            app_label = '.' + app_label
+        for s in settings.INSTALLED_APPS:
+            if s.endswith(app_label):
+                return True
+        #~ print "20120703 not installed: %r" % app_label
+
+        
 
 
     def get_quicklinks(self,ui,user):
-        from lino.utils import menus
-        tb = menus.Toolbar('quicklinks')
+        from lino.core import menus
+        tb = menus.Toolbar(user,'quicklinks')
         self.setup_quicklinks(ui,user,tb)
         return tb
         
     def get_site_menu(self,ui,user):
         from django.utils.translation import ugettext_lazy as _
-        from lino.utils import menus
-        main = menus.Toolbar('main')
+        from lino.core import menus
+        main = menus.Toolbar(user,'main')
         self.setup_menu(ui,user,main)
         main.compress()
         url = self.root_url
         if not url: 
             url = "/"
+        url = "javascript:Lino.close_all_windows()"
         main.add_url_button(url,label=_("Home"))
-        main.add_separator("->")
-        if user:
-            main.add_instance_action(user)
+        
+        #~ 20120626 if self.user_model:
+            #~ from lino.modlib.users import models as users
+            #~ main.add_separator("->")
+            #~ main.add_instance_action(user)
+            #~ main.add_action(users.MyDetail)
+            #~ main.add_action(users.Users.detail_action)
         
         return main
         
-    def setup_quicklinks(self,ui,user,tb):
-        """Override this 
+    def setup_quicklinks(self,ui,user,m):
+        """
+        Override this 
         in application-specific (or even local) :xfile:`settings.py` files 
         to define a series of *quick links* to appear below the main menu bar.
-        Example see :meth:`lino.apps.dsbe.settings.Lino.setup_quicklinks`.
+        Example see :meth:`lino.apps.pcsw.settings.Lino.setup_quicklinks`.
         """
-        pass
+        self.on_each_app('setup_quicklinks',ui,user,m)
+        
+    def setup_menu(self,ui,user,main):
+        """
+        Set up the application's menu structure.
+        
+        The default implementation use a system of 
+        predefined top-level items that are filled by the 
+        different :setting:`INSTALLED_APPS`.
+        
+        Deserves more documentation.
+        
+        """
+        from django.utils.translation import ugettext_lazy as _
+        m = main.add_menu("master",_("Master"))
+        self.on_each_app('setup_master_menu',ui,user,m)
+        if not user.profile.readonly:
+            m = main.add_menu("my",_("My menu"))
+            self.on_each_app('setup_my_menu',ui,user,m)
+        self.on_each_app('setup_main_menu',ui,user,main)
+        m = main.add_menu("config",_("Configure"))
+        self.on_each_app('setup_config_menu',ui,user,m)
+        m = main.add_menu("explorer",_("Explorer"))
+        self.on_each_app('setup_explorer_menu',ui,user,m)
+        m = main.add_menu("site",_("Site"))
+        self.on_each_app('setup_site_menu',ui,user,m)
+        return main
+
+
+    def on_each_app(self,methname,*args):
+        """
+        Call the named method on each module in :setting:`INSTALLED_APPS`
+        that defines it.
+        """
+        from django.conf import settings
+        from django.utils.importlib import import_module
+        from lino.utils import dblogger
+        
+        for app_name in settings.INSTALLED_APPS:
+            mod = import_module('.models', app_name)
+            meth = getattr(mod,methname,None)
+            if meth is not None:
+                dblogger.debug("Running %s of %s", methname, mod.__name__)
+                meth(self,*args)
         
-    def setup_menu(self,ui,user,menu):
-        raise NotImplementedError
         
     #~ def get_calendar_color(self,calendar,request):
         #~ if calendar.user == request.user:
@@ -838,11 +1149,15 @@ class Lino(object):
         if self.languages and len(self.languages) > 1:
             yield 'django.middleware.locale.LocaleMiddleware'
         #~ yield 'django.contrib.auth.middleware.AuthenticationMiddleware'
-        if self.user_model:
+        #~ if self.user_model:
+        if self.user_model is None:
+            yield 'lino.utils.auth.NoUserMiddleware'
+        elif self.remote_user_header:
             yield 'lino.utils.auth.RemoteUserMiddleware'
             yield 'django.middleware.doc.XViewMiddleware'
         else:
-            yield 'lino.utils.auth.NoUserMiddleware'
+            raise Exception("""\
+`user_model` is not None, but no `remote_user_header` in your settings.LINO.""")
         #~ yield 'lino.utils.editing.EditingMiddleware'
         yield 'lino.utils.ajax.AjaxExceptionResponse'
 
@@ -868,4 +1183,40 @@ class Lino(object):
         """
         return []
         
-                
\ No newline at end of file
+    def install_migrations(self,globals_dict):
+        """
+        Called from python dumps.
+        New in version 1.4.4. (replaces the previous migrations_module) 
+        """
+        #~ import logging
+        #~ dblogger = logging.getLogger(__name__)
+        from lino.utils import dblogger
+        from django.utils.importlib import import_module
+        if globals_dict['SOURCE_VERSION'] == __version__:
+            dblogger.info("Source version is %s : no migration needed", __version__)
+            return
+        if self.migration_module:
+            migmod = import_module(self.migration_module)
+        else:
+            migmod = self
+        while True:
+            from_version = globals_dict['SOURCE_VERSION']
+            funcname = 'migrate_from_' + from_version.replace('.','_')
+            m = getattr(migmod,funcname,None)
+            #~ func = globals().get(funcname,None)
+            if m:
+                #~ dblogger.info("Found %s()", funcname)
+                to_version = m(globals_dict)
+                if not isinstance(to_version,basestring):
+                    raise Exception("Oops: %s didn't return a string!" % m)
+                msg = "Migrating from version %s to %s" % (from_version, to_version)
+                if m.__doc__:
+                    msg += ":\n" + m.__doc__
+                dblogger.info(msg)
+                globals_dict['SOURCE_VERSION'] = to_version
+            else:
+                if from_version != __version__:
+                    dblogger.warning("No method for migrating from version %s to %s",from_version,__version__)
+                break
+
+          
